@@ -1,72 +1,125 @@
 package Module::Dependency::Info;
-
+use strict;
 use Storable qw/retrieve/;
 use vars qw/$VERSION $UNIFIED $unified_file $LOADED/;
+use constant MAX_DEPTH => 1000;
 
-($VERSION) = ('$Revision: 1.5 $' =~ /([\d\.]+)/ );
+$VERSION      = (q$Revision: 6562 $) =~ /(\d+)/g;
 $unified_file = '/var/tmp/dependence/unified.dat';
 
 sub setIndex {
-	my $file = shift;
-	TRACE("Trying to set index to <$file>");
-	return unless $file;
-	$unified_file = $file;
-	$LOADED = 0;
-	return 1;
+    my $file = shift;
+    TRACE("Trying to set index to <$file>");
+    return unless $file;
+    $unified_file = $file;
+    $LOADED       = 0;
+    return 1;
 }
 
 sub retrieveIndex {
-	TRACE("retrieving index");
-	$UNIFIED = retrieve( $unified_file ) || return(undef);
-	$LOADED = 1;
-	return $UNIFIED;
+    TRACE("retrieving index");
+    $UNIFIED = retrieve($unified_file) || return (undef);
+    $LOADED = 1;
+    return $UNIFIED;
 }
 
 sub allItems {
-	my $force = shift;
-	if (! $LOADED || $force) { retrieveIndex(); }
-	return [ keys %{$UNIFIED->{'allobjects'}} ];
+    my $force = shift;
+    if ( !$LOADED || $force ) { retrieveIndex(); }
+    return [ keys %{ $UNIFIED->{'allobjects'} } ];
 }
 
 sub allScripts {
-	my $force = shift;
-	if (! $LOADED || $force) { retrieveIndex(); }
-	return $UNIFIED->{'scripts'};
+    my $force = shift;
+    if ( !$LOADED || $force ) { retrieveIndex(); }
+    return $UNIFIED->{'scripts'};
 }
 
 sub getItem {
-	my ($packname, $force) = @_;
-	if (! $LOADED || $force) { retrieveIndex(); }
-	TRACE("Getting record for <$packname>");
-	if (exists $UNIFIED->{'allobjects'}->{$packname}) {
-		return $UNIFIED->{'allobjects'}->{$packname};
-	} else {
-		return undef;
-	}
+    my ( $packname, $force ) = @_;
+    if ( !$LOADED || $force ) { retrieveIndex(); }
+    TRACE("Getting record for <$packname>");
+    if ( exists $UNIFIED->{'allobjects'}->{$packname} ) {
+        return $UNIFIED->{'allobjects'}->{$packname};
+    }
+    else {
+        return undef;
+    }
 }
 
 sub getFilename {
-	my $obj = getItem( @_ ) || return(undef);
-	return $obj->{'filename'};
+    my $obj = getItem(@_) || return (undef);
+    return $obj->{'filename'};
 }
 
 sub getChildren {
-	my $obj = getItem( @_ ) || return(undef);
-	return $obj->{'depends_on'};
+    my $obj = getItem(@_) || return (undef);
+    return $obj->{'depends_on'};
 }
 
 sub getParents {
-	my $obj = getItem( @_ ) || return(undef);
-	return $obj->{'depended_upon_by'};
+    my $obj = getItem(@_) || return (undef);
+    return $obj->{'depended_upon_by'};
 }
 
 sub dropIndex {
-	$LOADED = 0;
-	undef $UNIFIED;
-	return 1;
+    $LOADED = 0;
+    undef $UNIFIED;
+    return 1;
 }
 
-sub TRACE {}
+sub relationship {
+    my ( $itemName, $otherItem ) = @_;
+    TRACE("relationship for $itemName / $otherItem");
+    my $obj = getItem($itemName) || return (undef);
+
+    my ( $isParent, $isChild ) =
+        ( _isParent( $itemName, $otherItem, {}, 0 ), _isChild( $itemName, $otherItem, {}, 0 ) );
+
+    my $rel;
+    if ( $isParent && $isChild ) { $rel = 'CIRCULAR'; }
+    elsif ($isParent) { $rel = 'PARENT'; }
+    elsif ($isChild)  { $rel = 'CHILD'; }
+    else { $rel = 'NONE'; }
+
+    return $rel;
+}
+
+### PRIVATE
+
+sub _isParent {
+    my ( $itemName, $otherItem, $seen, $depth ) = @_;
+    TRACE("_isParent for $itemName / $otherItem");
+    return 0 if $seen->{$itemName}++;
+    my $parents = getParents($itemName);
+    foreach (@$parents) {
+        return 1 if ( $_ eq $otherItem );
+    }
+    TRACE("...not directly, recursing");
+    foreach (@$parents) {
+        die "Deep recursion detected" if ( $depth > MAX_DEPTH );
+        return 1 if _isParent( $_, $otherItem, $seen, $depth++ );
+    }
+    return 0;
+}
+
+sub _isChild {
+    my ( $itemName, $otherItem, $seen, $depth ) = @_;
+    TRACE("_isChild for $itemName / $otherItem");
+    return 0 if $seen->{$itemName}++;
+    my $children = getChildren($itemName);
+    foreach (@$children) {
+        return 1 if ( $_ eq $otherItem );
+    }
+    TRACE("...not directly, recursing");
+    foreach (@$children) {
+        die "Deep recursion detected" if ( $depth > MAX_DEPTH );
+        return 1 if _isChild( $_, $otherItem, $seen, $depth++ );
+    }
+    return 0;
+}
+
+sub TRACE { }
 
 =head1 NAME
 
@@ -74,31 +127,34 @@ Module::Dependency::Info - retrieve dependency information for scripts and modul
 
 =head1 SYNOPSIS
 
-	use Module::Dependecy::Info;
-	Module::Dependecy::Info::setIndex( '/var/tmp/dependence/unified.dat' );
+	use Module::Dependency::Info;
+	Module::Dependency::Info::setIndex( '/var/tmp/dependence/unified.dat' );
 	
 	# load the index (actually it's loaded automatically if needed so this is optional)
-	Module::Dependecy::Info::retrieveIndex();
+	Module::Dependency::Info::retrieveIndex();
 	# or
-	$refToEntireDatabase = Module::Dependecy::Info::retrieveIndex();
+	$refToEntireDatabase = Module::Dependency::Info::retrieveIndex();
 	
 	$listref = Module::Dependency::Info::allItems();
 	$listref = Module::Dependency::Info::allScripts();
 	
 	# note the syntax here - the name of perl scripts, but the package name of modules.
-	$dependencyInfo = Module::Dependecy::Info::getItem( 'Foo::Bar' [, $forceReload ] );
+	$dependencyInfo = Module::Dependency::Info::getItem( 'Foo::Bar' [, $forceReload ] );
 	# and
-	$dependencyInfo = Module::Dependecy::Info::getItem( 'blahblah.pl' [, $forceReload ] );
+	$dependencyInfo = Module::Dependency::Info::getItem( 'blahblah.pl' [, $forceReload ] );
 	
-	$filename = Module::Dependecy::Info::getFilename( 'Foo::Bar' [, $forceReload ] );
-	$listref = Module::Dependecy::Info::getChildren( $node [, $forceReload ] );
-	$listref = Module::Dependecy::Info::getParents( $node [, $forceReload ] );
+	$filename = Module::Dependency::Info::getFilename( 'Foo::Bar' [, $forceReload ] );
+	$listref = Module::Dependency::Info::getChildren( $node [, $forceReload ] );
+	$listref = Module::Dependency::Info::getParents( $node [, $forceReload ] );
 	
-	Module::Dependecy::Info::dropIndex();
+	$value = Module::Dependency::Info::relationship( 'Foo::Bar', 'strict' );
+	
+	Module::Dependency::Info::dropIndex();
 
 =head1 DESCRIPTION
 
-This module is used to access the data structures created by Module::Dependecy::Indexer. 
+This module is used to access the data structures created by Module::Dependency::Indexer
+B<OR> a third-party application that creates databases of the correct format. 
 Although you can get at the database structure itself you should use the accessor methods.
 
 =head1 METHODS
@@ -126,7 +182,7 @@ keys() gives us. The entries in the array are things like 'foo.pl' and 'Bar::Baz
 Returns a reference ot an array of all the scripts in the currently loaded datafile. The order is whatever
 it is in the datafile.
 
-=item $record = Module::Dependecy::Info::getItem( $name [, $forceReload ] );
+=item $record = Module::Dependency::Info::getItem( $name [, $forceReload ] );
 
 Returns entire record for the thing you name, or undef if no such entry can be found 
 (remember modules are referred to like 'Foo::Bar' whereas scripts like 'foo.pl'). 
@@ -136,20 +192,36 @@ if you want to force a reload - this may be relevant in long-lived perl processe
 like mod_perl, but only do it when you need to, like every 10 minutes 
 or whatever makes sense for your application.
 
-=item $filename = Module::Dependecy::Info::getFilename( $node [, $forceReload ] );
+=item $filename = Module::Dependency::Info::getFilename( $node [, $forceReload ] );
 
 Gets the full filename for the package/script named, or undef if no record could be found.
 
-=item $listref = Module::Dependecy::Info::getChildren( $node [, $forceReload ] );
+=item $listref = Module::Dependency::Info::getChildren( $node [, $forceReload ] );
 
 Gets a list of all dependencies, i.e. packages that this item depends on, for the 
 package/script named, or undef if no record could be found.
 
-=item $listref = Module::Dependecy::Info::getParents( $node [, $forceReload ] );
+=item $listref = Module::Dependency::Info::getParents( $node [, $forceReload ] );
 
 Gets a list of all reverse dependencies, i.e. packages that depend upon this item, for 
 the package/script named, or undef if no record could be found.
 
+=item $value = Module::Dependency::Info::relationship( $itemName, $otherItem );
+
+Tells you whether, according to the current database, $itemName is related to $otherItem.
+$itemName is a module or script in the database (i.e. it's a file that has been indexed).
+Return values are:
+
+undef if $itemName is not in the database
+
+'NONE' if no link can be found (may be a false negative if links between the 2 items are not in the index)
+
+'PARENT' if the $otherItem depends upon $itemName
+
+'CHILD' if $itemName depends upon $otherItem
+
+'CIRCULAR' if $otherItem is both 'PARENT' and 'CHILD'.
+
 =item dropIndex
 
 drops the current database - you generally have no need to do this unless you're trying to save
@@ -211,6 +283,14 @@ But of course you should use the accessor methods to get at the information.
 There is a TRACE stub function, and the module uses TRACE() to log activity. Override our 
 TRACE with your own routine, e.g. one that prints to STDERR, to see these messages.
 
+=head1 SEE ALSO
+
+Module::Dependency and the README files.
+
+=head1 VERSION
+
+$Id: Info.pm,v 1.9 2002/09/25 23:06:35 piers Exp $
+
 =cut
 
 
